unit U_Graph;

interface

const MAXKNOTEN = 10;
      MAXKANTEN = 100;
type
  TKnoten= char;
  TKante = record
                Start, Ziel: char;
           end;
  TWeg = record
           Anzahl: integer;
           Kanten: array[1..MAXKANTEN] of TKante;
         end;

  TAdjazenzmatrix = array[1..MAXKNOTEN,1..MAXKNOTEN] of integer;

  TGraph = class

                Knotenanzahl: integer;
                Kantenanzahl: integer;
                Knoten: array[1..MAXKNOTEN] of TKnoten;
                Kanten: array[1..MAXKANTEN] of TKante;

                constructor create;

                procedure Oeffnen(Filename: string);
                procedure Speichern(Filename: string);
                function SucheEulerkreis: TWeg;
                function PruefeEulerkreis: boolean;
                function Adjazenzmatrix: TAdjazenzmatrix;
                function Tiefensuche: TWeg;
                function Hamilton: TWeg;
           end;


implementation

constructor TGraph.Create;
var i: integer;
begin
  inherited create;
  for i:=1 to MAXKNOTEN
  do Knoten[i]:= ' ';
  for i:= 1 to MAXKANTEN
  do begin
       Kanten[i].Start:= ' ';
       Kanten[i].Ziel:= ' ';
     end;
end;


procedure TGraph.Oeffnen(Filename: string);
var Datei: Textfile;
    i: integer;
    str: string;
begin
  assignfile(Datei,Filename);
  Reset(Datei);
  Readln(Datei,Knotenanzahl);
  for i:= 1 to Knotenanzahl
  do begin
       Readln(Datei,str);
       Knoten[i]:= str[1];
     end;
  Readln(Datei);
  Readln(Datei,Kantenanzahl);
  for i:= 1 to Kantenanzahl
  do begin
       readln(Datei,str);
       Kanten[i].Start:= str[1];
       Kanten[i].Ziel:= str[2];
     end;
  closefile(Datei);
end;

procedure TGraph.Speichern(Filename: string);
var Datei: Textfile;
    i: integer;
begin
  assignfile(Datei,Filename);
  ReWrite(Datei);
  writeln(Datei,Knotenanzahl);
  for i:= 1 to Knotenanzahl
  do writeln(Datei,Knoten[i]);
  writeln(Datei,'---------------------');
  writeln(Datei,Kantenanzahl);
  for i:= 1 to Kantenanzahl
  do writeln(Datei,Kanten[i].Start+Kanten[i].Ziel);
  closefile(Datei);
end;

function TGraph.SucheEulerkreis: TWeg;
var AktKnoten: TKnoten;
    KanteBesucht: array[1..MAXKANTEN] of boolean;
    AlleKantenBesucht: boolean;
    KantenNr: integer;
    i: integer;
begin
  result.Anzahl:= 0;
  for i:= 1 to MAXKANTEN
  do KanteBesucht[i]:= false;

  AktKnoten := Knoten[1];
  AlleKantenBesucht := (Kantenanzahl=0);
  While AlleKantenBesucht=false
  do begin
       { Suche Kante, welche vom aktuellen Knoten noch abgeht und nicht besucht wurde }
       KantenNr:= 0;
       for i := 1 to Kantenanzahl
       do begin
            if (KanteBesucht[i]=false) and
               ((Kanten[i].Start=AktKnoten) or (Kanten[i].Ziel=AktKnoten))
            then KantenNr:= i;
          end;
       if KantenNr > 0
       then begin      { Der Weg lt sich vom AktKnoten aus weiterfhren }
              inc(result.Anzahl);
              result.Kanten[result.Anzahl].Start:= AktKnoten;
              if AktKnoten<>Kanten[KantenNr].Ziel
              then AktKnoten:= Kanten[KantenNr].Ziel
              else AktKnoten:= Kanten[KantenNr].Start;
              result.Kanten[result.Anzahl].Ziel:= AktKnoten;
              KanteBesucht[KantenNr]:= true;
            end
       else begin      { Der Aktuelle Weg ist ein unvollstndiger Rundweg }
              { Starte den Rundweg von einem anderen Knoten ausgehend }
              result.Kanten[result.Anzahl+1]:= result.Kanten[1];
              for i:= 1 to result.Anzahl
              do result.Kanten[i]:= result.Kanten[i+1];
              AktKnoten:= result.Kanten[result.Anzahl].Ziel;
            end;
       { Testen, ob alle Kanten besucht sind }
       AlleKantenBesucht:= true;
       for i:= 1 to Kantenanzahl
       do if KanteBesucht[i]=false
          then AlleKantenBesucht:= false;
     end;
end;

function TGraph.PruefeEulerkreis: boolean;
var Knotennr, Kantennr: integer;
    Anzahl: integer;
begin
  result:= true;
  for Knotennr:= 1 to Knotenanzahl
  do begin
       Anzahl:= 0;
       for Kantennr:= 1 to Kantenanzahl
       do begin
            if (Kanten[Kantennr].Start=Knoten[Knotennr]) or
               (Kanten[Kantennr].Ziel=Knoten[Knotennr])
            then inc(Anzahl)
          end;
       if Anzahl mod 2 <> 0 then result:= false;
     end;
end;

function TGraph.Adjazenzmatrix: TAdjazenzmatrix;
var x,y,k: integer;
begin
  for x:= 1 to MAXKNOTEN
  do begin
       for y:= 1 to MAXKNOTEN
       do begin
            result[x,y]:= 0;
            for k:= 1 to Kantenanzahl
            do if ((Kanten[k].Start=Knoten[x]) and (Kanten[k].Ziel=Knoten[y])) or
                  ((Kanten[k].Ziel=Knoten[x]) and (Kanten[k].Start=Knoten[y]))
               then result[x,y]:= 1;
          end;
     end;
end;

function TGraph.Tiefensuche: TWeg;
var besucht: array[1..MAXKNOTEN] of boolean;
    Matrix: TAdjazenzmatrix;
    i: integer;
    Weg: TWeg;

  procedure SucheInDieTiefe(AktKnoten: integer);
  var Zielknoten: integer;
  begin
    for Zielknoten:= 1 to Knotenanzahl
    do if (not besucht[Zielknoten]) and (Matrix[AktKnoten,Zielknoten]>0)
       then begin
              inc(Weg.Anzahl);
              Weg.Kanten[Weg.Anzahl].Start:= Knoten[AktKnoten];
              Weg.Kanten[Weg.Anzahl].Ziel:= Knoten[Zielknoten];
              besucht[Zielknoten]:= true;
              SucheInDieTiefe(Zielknoten);
            end;
  end;

begin
  for i:= 1 to MAXKNOTEN
  do besucht[i]:= false;
  Weg.Anzahl:= 0;
  Matrix:= Adjazenzmatrix;

  for i:= 1 to Knotenanzahl
  do if not besucht[i]
     then begin
            besucht[i]:= true;
            SucheInDieTiefe(i);
          end;
  result:= Weg;
end;

function TGraph.Hamilton: TWeg;
var Matrix: TAdjazenzmatrix;
    Kanten: TWeg;
    Ergebnis: TWeg;
    BesuchteKnoten: array[1..MAXKNOTEN] of boolean;

    procedure Initialisierung;
    var i: integer;
    begin
      Matrix:= Adjazenzmatrix;
      for i:= 1 to Knotenanzahl
      do BesuchteKnoten[i]:= false;

      Kanten.Anzahl:= 0;
    end;

    procedure Suche_rekursiv(Nr: integer);
    var i: integer;
        fertig: boolean;
    begin
      BesuchteKnoten[Nr]:= true;
      fertig:= true;
      for i:= 1 to KnotenAnzahl
      do if not BesuchteKnoten[i]
         then fertig:= false;
      if fertig
      then begin
             if Matrix[Nr,1]>0
             then begin
                    Ergebnis:= Kanten;
                    inc(Ergebnis.Anzahl);
                    Ergebnis.Kanten[Ergebnis.Anzahl].Start:= Knoten[Nr];
                    Ergebnis.Kanten[Ergebnis.Anzahl].Ziel:= Knoten[1];
                  end;
           end
      else begin
             for i:= 1 to KnotenAnzahl
             do if (Matrix[Nr,i]>0) and (not BesuchteKnoten[i])
                then begin
                       inc(Kanten.Anzahl);
                       Kanten.Kanten[Kanten.Anzahl].Start:= Knoten[Nr];
                       Kanten.Kanten[Kanten.Anzahl].Ziel:= Knoten[i];
                       Suche_rekursiv(i);
                       dec(Kanten.Anzahl);
                     end;
           end;
      BesuchteKnoten[Nr]:= false;
    end;
begin
  Initialisierung;
  Suche_rekursiv(1);
  result:= Ergebnis;
end;


end.
